"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const conventionalRecommendedBump = require("conventional-recommended-bump");
const git = require("simple-git");
const GitHubApi = require("github");
const parseGithubUrl = require("parse-github-url");
const semver = require("semver");
const log_1 = require("./../log");
const read_file_1 = require("./../utilities/read-file");
async function collectInformation() {
    log_1.log('substep', 'Read the "package.json" file');
    const packageJson = await read_file_1.readFile('package.json');
    log_1.log('substep', 'Validate the "package.json" file');
    validatePackageJson(packageJson);
    const information = {};
    log_1.log('substep', 'Retrieve versioning details');
    information.isFirstVersion = !(await hasGitTags());
    information.oldVersion = packageJson.version;
    information.newVersion = information.isFirstVersion ? information.oldVersion : await getNewVersion(information.oldVersion);
    log_1.log('substep', 'Get repository information');
    const githubUrl = parseGithubUrl(packageJson.repository.url);
    information.repositoryOwner = githubUrl.owner;
    information.repositoryName = githubUrl.name;
    information.repositoryUrl = githubUrl.href;
    log_1.log('substep', 'Read & validate GitHub API token');
    information.githubToken = await getGithubToken(information.repositoryOwner, information.repositoryName);
    return information;
}
exports.collectInformation = collectInformation;
function validatePackageJson(packageJson) {
    if (!packageJson.hasOwnProperty('version')) {
        throw new Error(`The "package.json" file defines no version.`);
    }
    if (semver.valid(packageJson.version) === null) {
        throw new Error(`The "package.json" file defines the version "${packageJson.version}"; regarding semantic versioning this is not a valid version number.`);
    }
    if (semver.prerelease(packageJson.version) !== null) {
        throw new Error(`The "package.json" file defines the version "${packageJson.version}"; pre-release versions are not supported by automatic-release.`);
    }
    if (!packageJson.hasOwnProperty('repository') || !packageJson.repository.hasOwnProperty('url')) {
        throw new Error('The "package.json" file defines no repository URL.');
    }
}
function hasGitTags() {
    return new Promise((resolve, reject) => {
        git(process.cwd())
            .tags((gitTagsError, gitTags) => {
            if (gitTagsError) {
                reject(new Error(`An error has occured while retrieving the project's Git tags. [${gitTagsError.message}]`));
                return;
            }
            resolve(gitTags.all.length !== 0);
        });
    });
}
function getNewVersion(oldVersion) {
    return new Promise((resolve, reject) => {
        conventionalRecommendedBump({
            preset: 'angular'
        }, (error, recommendedBump) => {
            if (error) {
                reject(new Error(`An error occured while evaluating the next version. [${error.message}]`));
                return;
            }
            const newVersion = semver.inc(oldVersion, recommendedBump.releaseType);
            resolve(newVersion);
        });
    });
}
function getGithubToken(repositoryOwner, repositoryName) {
    return new Promise((resolve, reject) => {
        if (!process.env.hasOwnProperty('GH_TOKEN')) {
            reject(new Error('The "GH_TOKEN" environment variable is not set.'));
            return;
        }
        const githubToken = process.env.GH_TOKEN;
        const github = new GitHubApi({
            timeout: 5000
        });
        github.authenticate({
            type: 'oauth',
            token: githubToken
        });
        github.repos.getCollaborators({
            owner: repositoryOwner,
            repo: repositoryName
        }, (error, collaborators) => {
            if (error) {
                reject(new Error(`An error occured while verifying the GitHub token. [${error.headers.status}: "${JSON.parse(error.message).message}"]`));
                return;
            }
            resolve(githubToken);
        });
    });
}
//# sourceMappingURL=information.js.map